#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#from typing import *
from enum import IntEnum
from typing import Dict, List
from . IndexRange import *
from . StringList import *


class Context():
    """
    Contains global tables for strings, and lists-of-strings.
    It contains:

        - string_buffer: A list of null-terminated strings. The list is
          partitioned into contiguous segments dedicated to a specific
          kind of string, e.g.:
            - all instruction opcodes
            - all extension names
            - all aliases for a given enum
            - all enum names for a first operand type
            - all enum names for a second operand type, etc.
          Strings are sorted within each segment.

        - string_total_len: The sum of lengths of strings in string_buffer.

        - strings: Maps a string to an IndexRange indicating where the string
          can be found in the (future) concatenation of all strings in the
          string_buffer.

        - range_buffer: A dictionary mapping a string kind to a list IndexRange
              objects R. Each R is one of:

            - An index range referencing one string as it appears in the
              (future) concatenation of all strings in the string_buffer.
              In this case R represents a single string.

            - An index range referencing earlier elements in range_buffer[kind]
              itself.  In this case R represents a list of strings.

        - ranges:  A dictionary mapping a string kind and lists-of-strings
            to its encoding in the range_buffer array.

            It is a two-level mapping of Python type:

                Dict[str,dict[StringList,IndexRange]]

            where

                ranges[kind][list of strings] = an IndexRange

            The 'kind' string encodes a purpose including:
                - opcodes: the list of instruction opcode strings.
                - the list of aliases for an opcode, or an enum
                - an operand type such as 'SPV_OPERAND_TYPE_DIMENSIONALITY':
                  the list of operand type names, e.g. '2D', '3D', 'Cube',
                  'Rect', etc. in the case of SPV_OPERAND_TYPE_DIMENSIONALITY.
            By convention, the 'kind' string should be a singular noun for
            the type of object named by each member of the list.

            The IndexRange leaf value encodes a list of strings as in the
            second case described for 'range_buffer'.

    """
    def __init__(self) -> None:
        self.string_total_len: int = 0  # Sum of  lengths of all strings in string_buffer
        self.string_buffer: List[str] = []
        self.strings: Dict[str, IndexRange] = {}
        self.ir_to_string: Dict[IndexRange, str] = {} # Inverse of self.strings

        self.range_buffer: Dict[str,List[IndexRange]] = {}
        # We need StringList here because it's hashable, and so it
        # can be used as the key for a dict.
        self.ranges: Dict[str,Dict[StringList,IndexRange]] = {}

    def GetString(self, ir: IndexRange) -> str:
        if ir in self.ir_to_string:
            return self.ir_to_string[ir]
        raise Exception("unregistered index range {}".format(str(ir)))

    def AddString(self, s: str) -> IndexRange:
        """
        Adds or finds a string in the string_buffer.
        Returns its IndexRange.
        """
        if s in self.strings:
            return self.strings[s]
        # Allocate space, including for the terminating null.
        s_space: int = len(s) + 1
        ir = IndexRange(self.string_total_len, s_space)
        self.strings[s] = ir
        self.ir_to_string[ir] = s
        self.string_total_len += s_space
        self.string_buffer.append(s)
        return ir

    def AddStringList(self, kind: str, words: List[str]) -> IndexRange:
        """
        Ensures a list of strings is recorded in range_buffer[kind], and
        returns its location in the range_buffer[kind].
        As a side effect, also ensures each string in the list is in
        the string_buffer.
        """
        l = StringList(words)

        entry: Dict[StringList, IndexRange] = self.ranges.get(kind, {})
        if kind not in self.ranges:
            self.ranges[kind] = entry
            self.range_buffer[kind] = []

        if l in entry:
            return entry[l]
        new_ranges = [self.AddString(s) for s in l]
        ir = IndexRange(len(self.range_buffer[kind]), len(new_ranges))
        self.range_buffer[kind].extend(new_ranges)
        entry[l] = ir
        return ir

    def dump(self) -> None:
        print("string_total_len: {}".format(self.string_total_len))

        sbi = 0
        print("string_buffer:")
        for sb in self.string_buffer:
            print("  {}: '{}'".format(sbi, sb))
            sbi += len(sb) + 1
        print("")

        s = []
        for k,v in self.strings.items():
            s.append("'{}': {}".format(k,str(v)))
        print("strings:\n  {}\n".format('\n  '.join(s)))

        for rbk, rbv in self.range_buffer.items():
            print("range_buffer[{}]:".format(rbk))
            i: int = 0
            for r in rbv:
                print("  {} {}: {}".format(rbk, i, str(r)))
                i += 1
            print("")

        for rk, rv in self.ranges.items():
            for key,val in rv.items():
                print("ranges[{}][{}]: {}".format(str(rk),str(key), str(val)))
        print("")





#!/usr/bin/env python3

# Copyright (c) 2016 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Updates an output file with version info unless the new content is the same
# as the existing content.
#
# Args: <changes-file> <output-file>
#
# The output file will contain a line of text consisting of two C source syntax
# string literals separated by a comma:
#  - The software version deduced from the given CHANGES file.
#  - A longer string with the project name, the software version number, and
#    git commit information for the CHANGES file's directory.  The commit
#    information is the content of the FORCED_BUILD_VERSION_DESCRIPTION
#    environement variable is it exists, else the output of "git describe" if
#    that succeeds, or "git rev-parse HEAD" if that succeeds, or otherwise a
#    message containing the phrase "unknown hash".
# The string contents are escaped as necessary.

import datetime
import errno
import os
import os.path
import re
import subprocess
import logging
import sys
import time

# Format of the output generated by this script. Example:
# "v2023.1", "SPIRV-Tools v2023.1 0fc5526f2b01a0cc89192c10cf8bef77f1007a62, 2023-01-18T14:51:49"
OUTPUT_FORMAT = '"{version_tag}", "SPIRV-Tools {version_tag} {description}"\n'

def mkdir_p(directory):
    """Make the directory, and all its ancestors as required.  Any of the
    directories are allowed to already exist."""

    if directory == "":
        # We're being asked to make the current directory.
        return

    try:
        os.makedirs(directory)
    except OSError as e:
        if e.errno == errno.EEXIST and os.path.isdir(directory):
            pass
        else:
            raise

def command_output(cmd, directory):
    """Runs a command in a directory and returns its standard output stream.

    Returns (False, None) if the command fails to launch or otherwise fails.
    """
    try:
      # Set shell=True on Windows so that Chromium's git.bat can be found when
      # 'git' is invoked.
      p = subprocess.Popen(cmd,
                           cwd=directory,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE,
                           shell=os.name == 'nt')
      (stdout, _) = p.communicate()
      if p.returncode != 0:
        return False, None
    except Exception as e:
        return False, None
    return p.returncode == 0, stdout

def deduce_software_version(changes_file):
    """Returns a tuple (success, software version number) parsed from the
    given CHANGES file.

    Success is set to True if the software version could be deduced.
    Software version is undefined if success if False.
    Function expects the CHANGES file to describes most recent versions first.
    """

    # Match the first well-formed version-and-date line
    # Allow trailing whitespace in the checked-out source code has
    # unexpected carriage returns on a linefeed-only system such as
    # Linux.
    pattern = re.compile(r'^(v\d+\.\d+(-dev)?) \d\d\d\d-\d\d-\d\d\s*$')
    with open(changes_file, mode='r') as f:
        for line in f.readlines():
            match = pattern.match(line)
            if match:
                return True, match.group(1)
    return False, None


def describe(repo_path):
    """Returns a string describing the current Git HEAD version as descriptively
    as possible.

    Runs 'git describe', or alternately 'git rev-parse HEAD', in directory.  If
    successful, returns the output; otherwise returns 'unknown hash, <date>'."""

    # if we're in a git repository, attempt to extract version info
    success, output = command_output(["git", "rev-parse", "--show-toplevel"], repo_path)
    if success:
        success, output = command_output(["git", "describe", "--tags", "--match=v*", "--long"], repo_path)
        if not success:
            success, output = command_output(["git", "rev-parse", "HEAD"], repo_path)

        if success:
            # decode() is needed here for Python3 compatibility. In Python2,
            # str and bytes are the same type, but not in Python3.
            # Popen.communicate() returns a bytes instance, which needs to be
            # decoded into text data first in Python3. And this decode() won't
            # hurt Python2.
            return output.rstrip().decode()

    # This is the fallback case where git gives us no information,
    # e.g. because the source tree might not be in a git tree or
    # git is not available on the system.
    # In this case, usually use a timestamp.  However, to ensure
    # reproducible builds, allow the builder to override the wall
    # clock time with environment variable SOURCE_DATE_EPOCH
    # containing a (presumably) fixed timestamp.
    timestamp = int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
    iso_date = datetime.datetime.fromtimestamp(timestamp, datetime.timezone.utc).isoformat()
    return "unknown hash, {}".format(iso_date)

def main():
    FORMAT = '%(asctime)s %(message)s'
    logging.basicConfig(format="[%(asctime)s][%(levelname)-8s] %(message)s", datefmt="%H:%M:%S")
    if len(sys.argv) != 3:
        logging.error("usage: {} <repo-path> <output-file>".format(sys.argv[0]))
        sys.exit(1)

    changes_file_path = os.path.realpath(sys.argv[1])
    output_file_path = sys.argv[2]

    success, version = deduce_software_version(changes_file_path)
    if not success:
      logging.error("Could not deduce latest release version from {}.".format(changes_file_path))
      sys.exit(1)

    repo_path = os.path.dirname(changes_file_path)
    description = os.getenv("FORCED_BUILD_VERSION_DESCRIPTION", describe(repo_path))
    content = OUTPUT_FORMAT.format(version_tag=version, description=description)

    # Escape file content.
    content.replace('"', '\\"')

    if os.path.isfile(output_file_path):
      with open(output_file_path, 'r') as f:
        if content == f.read():
          return

    mkdir_p(os.path.dirname(output_file_path))
    with open(output_file_path, 'w') as f:
        f.write(content)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from typing import List

class StringList(list):
    """
    A hashable ordered list of strings.
    This can be used as the key for a dictionary.
    """
    def __init__(self, strs: List[str]) -> None:
        super().__init__(strs)

    def __hash__(self) -> int: # type: ignore[override]
        return functools.reduce(lambda h, ir: hash((h, hash(ir))), self, 0)



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Any

class IndexRange():
  def __init__(self, first: int, count: int) -> None:
    self.first = first
    self.count = count
    if first < 0:
      raise Exception("invalid arg: first {} must be non-negative".format(first))
    if count < 0:
      raise Exception("invalid arg: count {} must be non-negative".format(count))

  def __eq__(self, other: Any) -> bool:
    return isinstance(other, IndexRange) and self.first == other.first and self.count == other.count

  def __hash__(self) -> int:
    return hash("{} {}".format(self.first, self.count))

  def __str__(self) -> str:
    return "IR({}, {})".format(self.first, self.count)



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from . Operand import *

class TestOperand(unittest.TestCase):
    def test_enumerant(self) -> None:
        x = Operand({'enumerant': 'abc'});
        self.assertEqual(x.enumerant, 'abc');

    def test_value_decimal(self) -> None:
        x = Operand({'value': 123});
        self.assertEqual(x.value, 123);

    def test_value_str_hex(self) -> None:
        x = Operand({'value': "0x0101"});
        self.assertEqual(x.value, 257);

    def test_value_str_dec(self) -> None:
        x = Operand({'value': "0101"});
        self.assertEqual(x.value, 101);

    def test_value_str_invalid_dec(self) -> None:
        x = Operand({'value': "01ab"});
        self.assertRaises(Exception, lambda y: x.value, 0);

    def test_value_str_invalid_hex(self) -> None:
        x = Operand({'value': "0x010j"});
        self.assertRaises(Exception, lambda y: x.value, 0);

    def test_capabilities_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.capabilities, []);

    def test_capabilities_present(self) -> None:
        x = Operand({'capabilities': ['abc', 'def']});
        self.assertEqual(x.capabilities, ['abc', 'def']);

    def test_extensions_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.extensions, []);

    def test_extensions_present(self) -> None:
        x = Operand({'extensions': ['abc', 'def']});
        self.assertEqual(x.extensions, ['abc', 'def']);

    def test_aliases_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.aliases, []);

    def test_aliases_present(self) -> None:
        x = Operand({'aliases': ['abc', 'def']});
        self.assertEqual(x.aliases, ['abc', 'def']);

    def test_parameters_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.parameters, []);

    def test_parameters_present(self) -> None:
        x = Operand({'parameters': ['abc', 'def']});
        self.assertEqual(x.parameters, ['abc', 'def']);

    def test_version_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.version, None);

    def test_version_present(self) -> None:
        x = Operand({'version': '1.0'});
        self.assertEqual(x.version, '1.0');

    def test_lastVersion_absent(self) -> None:
        x = Operand({});
        self.assertEqual(x.lastVersion, None);

    def test_lastVersion_present(self) -> None:
        x = Operand({'lastVersion': '1.3'});
        self.assertEqual(x.lastVersion, '1.3');

    def test_all_propertites(self) -> None:
        x = Operand({'enumerant': 'Foobar',
                         'value': 12,
                         'capabilities': ["yes"],
                         'extensions': ["SPV_FOOBAR_baz_bat"],
                         'version': "1.0",
                         'lastVersion': "1.3",
                         });
        self.assertEqual(x.enumerant, 'Foobar');
        self.assertEqual(x.value, 12);
        self.assertEqual(x.capabilities, ["yes"]);
        self.assertEqual(x.extensions, ["SPV_FOOBAR_baz_bat"]);
        self.assertEqual(x.version, '1.0');
        self.assertEqual(x.lastVersion, '1.3');



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import List

class Operand():
    def __init__(self, json: dict) -> None:
        val = json.get('value',None)

        self._obj = json

    @property
    def enumerant(self) -> str:
        result = self._obj.get('enumerant', None)
        if result is None:
            raise Exception("operand needs an enumerant string")
        return result

    @property
    def value(self) -> int:
        val: str|int = self._obj['value']
        if isinstance(val, int):
            return val
        elif isinstance(val,str):
            if val.startswith("0x"):
                return int(val, 16)
            else:
                return int(val, 10)
        else:
            raise Exception("operand needs a value integer or string")

    @property
    def capabilities(self) -> List[str]:
        return self._obj.get('capabilities',[])

    @property
    def extensions(self) -> List[str]:
        return self._obj.get('extensions',[])

    @property
    def aliases(self) -> List[str]:
        return self._obj.get('aliases',[])

    @property
    def parameters(self) -> List[dict]:
        return self._obj.get('parameters',[])

    @property
    def version(self): #   -> str | None  # Invald in Python 3.8
        return self._obj.get('version',None)

    @property
    def lastVersion(self): #  -> str | None # Invalid in Python 3.8
        return self._obj.get('lastVersion',None)



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from . Context import Context
from . IndexRange import IndexRange
from . StringList import StringList

class TestCreate(unittest.TestCase):
  def test_creation(self) -> None:
    x = Context()
    self.assertIsInstance(x.string_total_len, int)
    self.assertIsInstance(x.string_buffer, list)
    self.assertIsInstance(x.strings, dict)
    self.assertEqual(x.string_total_len, 0)
    self.assertEqual(x.string_buffer, [])
    self.assertEqual(x.strings, {})

class TestString(unittest.TestCase):
  def test_AddString_new(self) -> None:
    x = Context()
    abc_ir = x.AddString("abc")
    self.assertEqual(abc_ir, IndexRange(0,4))
    self.assertEqual(x.string_total_len, 4)
    self.assertEqual(x.string_buffer, ["abc"])
    self.assertEqual(x.strings, {"abc": IndexRange(0,4)})

    qz_ir = x.AddString("qz")
    self.assertEqual(qz_ir, IndexRange(4,3))
    self.assertEqual(x.string_total_len, 7)
    self.assertEqual(x.string_buffer, ["abc", "qz"])
    self.assertEqual(x.strings, {"abc": IndexRange(0,4), "qz": IndexRange(4,3)})

    empty_ir = x.AddString("")
    self.assertEqual(empty_ir, IndexRange(7,1))
    self.assertEqual(x.string_total_len, 8)
    self.assertEqual(x.string_buffer, ["abc", "qz", ""])
    self.assertEqual(x.strings, {"abc": IndexRange(0,4), "qz": IndexRange(4,3), "": IndexRange(7,1)})

  def test_AddString_idempotent(self) -> None:
    x = Context()
    abc_ir = x.AddString("abc")
    self.assertEqual(abc_ir, IndexRange(0,4))
    self.assertEqual(x.string_total_len, 4)
    self.assertEqual(x.string_buffer, ["abc"])
    self.assertEqual(x.strings, {"abc": IndexRange(0,4)})

    abc_ir = x.AddString("abc")
    self.assertEqual(abc_ir, IndexRange(0,4))
    self.assertEqual(x.string_total_len, 4)
    self.assertEqual(x.string_buffer, ["abc"])
    self.assertEqual(x.strings, {"abc": IndexRange(0,4)})

class TestStringList(unittest.TestCase):
  def test_AddStringList_empty(self) -> None:
    x = Context()
    x_ir = x.AddStringList('x', [])
    self.assertEqual(x_ir, IndexRange(0,0))
    self.assertEqual(x.string_buffer, [])
    self.assertEqual(x.range_buffer, { 'x': [] })
    self.assertEqual(x.ranges, {'x': {StringList([]): IndexRange(0,0)}})

  def test_AddgStringList_nonempty(self) -> None:
    x = Context()
    x_ir = x.AddStringList('x', ["abc", "def"])

    self.assertEqual(x_ir, IndexRange(0,2))
    self.assertEqual(x.range_buffer, {'x': [IndexRange(0,4), IndexRange(4,4)]})
    self.assertEqual(x.ranges, {'x': {StringList(['abc','def']): IndexRange(0,2)}})

  def test_AddgStringList_nonempty_idempotent(self) -> None:
    x = Context()
    x_ir = x.AddStringList('x', ["abc", "def"])
    y_ir = x.AddStringList('x', ["abc", "def"])

    self.assertEqual(x_ir, IndexRange(0,2))
    self.assertEqual(y_ir, IndexRange(0,2))
    self.assertEqual(x.range_buffer, {'x': [IndexRange(0,4), IndexRange(4,4)]})
    self.assertEqual(x.ranges, {'x': {StringList(['abc','def']): IndexRange(0,2)}})

  def test_AddgStringList_nonempty_does_not_sort(self) -> None:
    x = Context()
    x_ir = x.AddStringList('x', ["abc", "def"])
    y_ir = x.AddStringList('x', ["def", "abc"])

    self.assertEqual(x_ir, IndexRange(0,2))
    self.assertEqual(y_ir, IndexRange(2,2))
    self.assertEqual(x.range_buffer, {'x': [IndexRange(0,4),
                                            IndexRange(4,4),
                                            IndexRange(4,4),
                                            IndexRange(0,4)]})
    self.assertEqual(x.ranges, {'x':{StringList(['abc','def']): IndexRange(0,2),
                                     StringList(['def','abc']): IndexRange(2,2)}})

  def test_AddgStringList_separate_by_kind(self) -> None:
    x = Context()
    x_ir = x.AddStringList('x', ["abc", "def"])
    y_ir = x.AddStringList('y', ["ghi", "abc"])

    self.assertEqual(x_ir, IndexRange(0,2))
    self.assertEqual(y_ir, IndexRange(0,2))
    self.assertEqual(x.range_buffer,
                     {'x': [IndexRange(0,4), IndexRange(4,4)],
                      'y': [IndexRange(8,4), IndexRange(0,4)]})
    self.assertEqual(x.ranges, {'x': {StringList(['abc','def']): IndexRange(0,2)},
                                'y': {StringList(['ghi','abc']): IndexRange(0,2)}})

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# Copyright 2025 Google LLC

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from . IndexRange import IndexRange

class TestIndexRange(unittest.TestCase):
  def test_creation(self) -> None:
    x: IndexRange = IndexRange(4,5);
    self.assertEqual(x.first, 4)
    self.assertEqual(x.count, 5)

  def test_creation_bad_first(self) -> None:
    self.assertRaises(Exception, IndexRange, -1, 5)

  def test_creation_bad_count(self) -> None:
    self.assertRaises(Exception, IndexRange, 1, -5)

  def test_distinct(self) -> None:
    x = IndexRange(4, 5);
    y = IndexRange(6, 7);
    self.assertNotEqual(x.first, y.first)
    self.assertNotEqual(x.count, y.count)

  def test_equality(self) -> None:
    self.assertEqual(IndexRange(4,5), IndexRange(4,5))
    self.assertNotEqual(IndexRange(4,5), IndexRange(4,7))
    self.assertNotEqual(IndexRange(7,5), IndexRange(4,5))

  def test_hash_heuristic(self) -> None:
    x = hash(IndexRange(4,5))
    y = hash(IndexRange(4,5))
    z = hash(IndexRange(6,7))
    self.assertEqual(x, y)
    self.assertNotEqual(x, z)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# Copyright (c) 2023 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Args: <CHANGES-file> <tag> <output-file>
# Updates an output file with changelog from the given CHANGES file and tag.
#  - search for first line matching <tag> in file <CHANGES-file>
#  - search for the next line with a tag
#  - writes all the lines in between those 2 tags into <output-file>

import errno
import os
import os.path
import re
import subprocess
import logging
import sys

# Regex to match the SPIR-V version tag.
# Example of matching tags:
#  - v2020.1
#  - v2020.1-dev
#  - v2020.1.rc1
VERSION_REGEX = re.compile(r'^(v\d+\.\d+) +[0-9]+-[0-9]+-[0-9]+$')

def mkdir_p(directory):
    """Make the directory, and all its ancestors as required.  Any of the
    directories are allowed to already exist."""

    if directory == "":
        # We're being asked to make the current directory.
        return

    try:
        os.makedirs(directory)
    except OSError as e:
        if e.errno == errno.EEXIST and os.path.isdir(directory):
            pass
        else:
            raise

def main():
    FORMAT = '%(asctime)s %(message)s'
    logging.basicConfig(format="[%(asctime)s][%(levelname)-8s] %(message)s", datefmt="%H:%M:%S")
    if len(sys.argv) != 4:
        logging.error("usage: {} <CHANGES-path> <tag> <output-file>".format(sys.argv[0]))
        sys.exit(1)

    changes_path = sys.argv[1]
    start_tag = sys.argv[2]
    output_file_path = sys.argv[3]

    changelog = []
    has_found_start = False
    with open(changes_path, "r") as file:
      for line in file.readlines():
        m = VERSION_REGEX.match(line)
        if m:
          print(m.groups()[0])
          print(start_tag)
          if has_found_start:
            break;
          if start_tag == m.groups()[0]:
            has_found_start = True
          continue

        if has_found_start:
          changelog.append(line)

    if not has_found_start:
      logging.error("No tag matching {} found.".format(start_tag))
      sys.exit(1)

    content = "".join(changelog)
    if os.path.isfile(output_file_path):
      with open(output_file_path, 'r') as f:
        if content == f.read():
          sys.exit(0)

    mkdir_p(os.path.dirname(output_file_path))
    with open(output_file_path, 'w') as f:
        f.write(content)
    sys.exit(0)

if __name__ == '__main__':
    main()
